#!/usr/bin/wish

#adapted from Tcl'ers wiki: http://wiki.tcl.tk/3060

namespace eval balloon {set last 0 ; namespace export balloon}

 proc ::balloon::balloon {args} {
   variable last
   variable tips

   set numArgs [llength $args]
   if { $numArgs < 1 || $numArgs > 2 } {
        return -code error "wrong # args: should be \"balloon widget ?text?\"";
      }

   set w [lindex $args 0]
   if { ![winfo exists $w] } {
        return -code error "bad window path name \"$w\""
      }

   if { [winfo class $w] == "Toplevel" } {
        return -code error "cannot create tooltip for toplevel windows";
      }

   if { $numArgs == "1" } {
        if { [info exists tips($w)] } {
             return $tips($w);
           } else {
             return "";
           }
      }

   set text [lindex $args 1]

   if { $text == "" } {
        # turn off tooltip
        if { [set x [lsearch [bindtags $w] "Balloon"]] >= 0 } {
             bindtags $w [lreplace [bindtags $w] $x $x]
           }
        unset -nocomplain tips($w)
        trace remove command $w delete ::balloon::autoclear
        return;
      }

   # OK, set up a (new?) tooltip

   if { [lsearch [bindtags $w] "Balloon"] < 0 } {
        bindtags $w [linsert [bindtags $w] 0 "Balloon"]
      }

   if { [lsearch [trace info command $w] {delete ::balloon::autoclear}] < 0 } {
        trace add command $w delete ::balloon::autoclear
      }

   set tips($w) $text

 };# balloon::balloon

 proc ::balloon::show {w} {
    variable tips
    if { ![info exists tips($w)] } {return}
    if {[eval winfo containing [winfo pointerxy .]]!=$w} {return}
     set top "$w.balloon"
     catch {destroy $top}
     toplevel $top -bd 1 -bg black
     wm overrideredirect $top 1
    if {[string equal [tk windowingsystem] aqua]}  {
	::tk::unsupported::MacWindowStyle style $top help none
    }
     pack [message $top.txt -aspect 10000 -bg lightyellow \
             -text $tips($w)]
     set wmx [winfo rootx $w]
     set wmy [expr [winfo rooty $w]+[winfo height $w]]
     wm geometry $top \
       [winfo reqwidth $top.txt]x[winfo reqheight $top.txt]+$wmx+$wmy
     raise $top
 };# balloon::show

 proc ::balloon::show8.5 {w} {
    variable tips
    if { ![info exists tips($w)] } {return}
   # if {[winfo containing {expand}[winfo pointerxy .]]!= $w} {return}
     set top "$w.balloon"
     catch {destroy $top}
     toplevel $top -bd 1 -bg black
     bind $top <Button-1> [list destroy $top]
     wm overrideredirect $top 1
    if {[string equal [tk windowingsystem] aqua]}  {
	::tk::unsupported::MacWindowStyle style $top help none
    }
     pack [message $top.txt -aspect 10000 -bg lightyellow \
             -text $tips($w)]
     set wmx [winfo rootx $w]
     set wmy [expr [winfo rooty $w]+[winfo height $w]]
     wm geometry $top \
       [winfo reqwidth $top.txt]x[winfo reqheight $top.txt]+$wmx+$wmy
     raise $top
 };# balloon::show8.5

# if { [package vcompare [package require Tcl] 8.5] >= 0 } {
 #     rename ::balloon::show8.5 ::balloon::show
 #   } else {
 #     rename ::balloon::show8.5 {}
 #   }

 proc ::balloon::delay {} {
   variable last

   set then $last
   set last [clock seconds]
   if { [expr {$last - $then}] < 3} {
        return 50
      } else {
        return 1000
      }

 };# balloon::delay

 proc ::balloon::autoclear {old new op} {
   variable tips

   unset -nocomplain tips([namespace tail $old]);

 };# balloon::autoclear

 namespace import ::balloon::balloon
 bind Balloon <Enter> {after [::balloon::delay] [list ::balloon::show %W]}
 bind Balloon <Leave> {destroy %W.balloon}

 #console show

 ##############
 # test script
# ##############
#button .b1 -text "First"  ; balloon .b1 "1st"
#button .b2 -text "Second" ; balloon .b2 "2nd"
#button .b3 -text "Third"  ; balloon .b3 "3rd"
#button .b4 -text "No tip"
#
#pack .b1 .b2 .b3 .b4 -side left -padx 4
#
# #uncomment below to clear the tooltip for "Third" button
# balloon .b3 "