#!/bin/sh
# here is a sample html viewer to demonstrate the library usage
# Copyright (c) 1995 by Sun Microsystems
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# This REQUIRES Tk4.0 -- make sure "wish" on the next line is a 4.0 version
# The next line is a TK comment, but a shell command \
#   exec wish4.0 -f "$0" "$@" & exit 0

# if {$tk_version < 4.0 || [regexp {b[123]} $tk_patchLevel] } {
# 	puts stderr "This library requires TK4.0, this is only $tk_version, \
# 			patchlevel $tk_patchLevel"
# 	exit 1
# }
# if {[catch {array get env *}]} {
# 	puts stderr "This library requires tcl7.4, this version is too old!"
# 	exit 1
# }

#additional modifications for HelpBrowser 1.0 package by Kevin Walzer, WordTech Software (c) 2006 WordTech Communications LLC

set argv0 ""
#puts stderr "Starting sample HTML viewer..."
##KW:  add support for balloon help, tile widgets

source [file join [file dirname [info script]] htmllib.tcl]
catch {source [file join [file dirname [info script]] balloon.tcl]}
# package require tile
# if { [string equal [tk windowingsystem] x11]} {
#     style theme use clam
# }

#KW: hovering effect
proc float {w} {
    bind $w <Any-Enter> "$w configure -relief groove"
    bind $w <Any-Leave> "$w configure -relief flat"
}


#KW: grab focus for difficult widgets
proc hover {w} {
    bind $w <Any-Enter> "focus -force $w"
    bind $w <Any-Leave> "lower $w"
}

#KW: button effects for labels
proc buttonPress {w} {
    $w configure  -relief sunken
}

proc buttonLift {w} {
    $w  configure -relief flat
   
}

# construct a simple user interface

proc setupBrowser {} {
    toplevel .help
    wm title .help "iReveal Help"

    
    frame .help.frame
    #	pack .he.lp.frame -side bottom -fill both -expand yes
    #menubutton .menu -relief raised -bd 2 -text options... -menu .menu.m
    label .help.quit   -text Dismiss -image depart -compound top
    float .help.quit
    bind .help.quit <Button-1> "buttonPress .help.quit; destroy .help"
    bind .help.quit <ButtonRelease-1> "buttonLift .help.quit"
    balloon .help.quit "Close the help viewer"


#-command "destroy .help"
 label .help.back  -text Back -image back -compound top


   label  .help.home -text Home -image home -compound top
    float .help.home
    bind .help.home <Button-1> "buttonPress .help.home; goHome"
    bind .help.home <ButtonRelease-1> "buttonLift .help.home"
    balloon .help.home "Reload the home page"

	#entry .entry  -textvariable Url -width 35
	#label .file  -text file:
	#label .status -textvariable Running -width 6 -relief ridge \
			-bd 2 -padx 9 -pady 3
	#label .msg -textvariable message
	scrollbar .help.scrollbar  -command ".help.text yview"  -orient v
	option add *Text.height 40 startup
	option add *Text.width 80 startup
	text .help.text  -yscrollcommand ".help.scrollbar set" -padx 3 -pady 3 -takefocus 0 -borderwidth 0 -highlightthickness .5
	.help.text configure -cursor hand2

	pack .help.frame -side bottom -fill both -expand yes
	pack .help.scrollbar -side right -expand 0 -fill y
	pack .help.text -side left -fill both -expand 1
	pack .help.back .help.home   .help.quit -in .help.frame -side left

	# set up some sample keyboard bindings for the text widget
	#bind .entry <Return> {render $Url}
	bind all <End> {.help.text yview end}
	bind all <Home> {.help.text yview 0.0}
	bind all <Next> {.help.text yview scroll 1 page}
	bind all <Prior> {.help.text yview scroll -1 page}

}

	# I'm constantly being criticized for never using menus.
	# so here's a menu.  So there.
	#menu .menu.m
	#.menu.m add command -label "option menu"
	#.menu.m add separator
	#.menu.m add command -label "font size" -foreground red 
	#.menu.m add radiobutton -label small -value 0   -variable Size \
	#	-command {HMset_state .text -size $Size; render $Url}
	#.menu.m add radiobutton -label medium -value 4  -variable Size \
	#	-command {HMset_state .text -size $Size; render $Url}
	#.menu.m add radiobutton -label large -value 12  -variable Size \
	#	-command {HMset_state .text -size $Size; render $Url}
	#.menu.m add separator
	#.menu.m add command -label "indent level" -foreground red
	#.menu.m add radiobutton -label small -value 0.6 -variable Indent \
	#	-command {HMset_indent .text $Indent}
	#.menu.m add radiobutton -label medium -value 1.2 -variable Indent \
	#	-command {HMset_indent .text $Indent}
	#.menu.m add radiobutton -label large -value 2.4 -variable Indent \
	#	-command {HMset_indent .text $Indent}


# Go render a page.  We have to make sure we don't render one page while
# still rendering the previous one.  If we get here from a recursive 
# invocation of the event loop, cancel whatever we were rendering when
# we were called.
# If we have a fragment name, try to go there.

proc render {file} {
    global HM.text Url HMlast
    global Running message

    #KW: configure text widget to read read/display page
    .help.text configure -state normal

    #KW: HMlast refers to list of previously viewed pages
    lappend HMlast $file
    puts [llength $HMlast]
    puts $HMlast

    #KW: configure "back button" to disabled
    if {[llength $HMlast] <= 1} {
	.help.back configure -relief flat -state disabled 
	bind .help.back <Button-1> ""
	bind .help.back <ButtonRelease-1> ""
	bind .help.back <Any-Enter> ".help.back configure -relief flat"
	bind .help.back <Any-Leave> ".help.back configure -relief flat"
	balloon .help.back ""

    } else {
	#KW: configure "back button" to normal
	float .help.back
	.help.back configure -state normal
	bind .help.back <Button-1> "buttonPress .help.back; renderLast"
	bind .help.back <ButtonRelease-1> "buttonLift .help.back"
	balloon .help.back "View the previous page"
    }

    set fragment ""
    regexp {([^#]*)#(.+)} $file dummy file fragment
    if {$file == "" && $fragment != ""} {
	HMgoto .help.text $fragment
	return
    }
    HMreset_win .help.text
    #	set Running busy
    #	set message "Displaying $file"
    update idletasks
    if {$fragment != ""} {
	HMgoto help.text $fragment
    }
    set Url $file
    HMparse_html [get_html $file] {HMrender .help.text}
    set Running ready
    HMset_state .help.text -stop 1	;# stop rendering previous page if busy
    set message ""

    #reset text widget to disabled--prevents user from editing text in help display
    .help.text configure -state disabled
}

proc renderLast {} {

    #KW: procedure courtesy of Bruce Johnson, http://www.onemoonscientific.com/


    global HMlast
    
    set len [llength $HMlast]
    set newURL [lindex $HMlast [expr $len-2]]
    set HMlast [lrange $HMlast 0 [expr $len-3]]
    
    
    render $newURL

}


proc goHome {} {
    global Home
    render $Home
}

# given a file name, return its html, or invent some html if the file can't
# be opened.

proc get_html {file} {
	global Home

	if {[catch {set fd [open $file]} msg]} {
		return "
			<title>Bad file $file</title>
			<h1>Error reading $file</h1><p>
			$msg<hr>
			<a href=$Home>Go home</a>
	       "
	}

	set result [read $fd]
	close $fd
	return $result
}

# Override the library link-callback routine for the sample app.
# It only handles the simple cases.

proc HMlink_callback {win href} {
    puts $href
	global Url 

    ##KW: add handlers for standard types of extensions--http, ftp, mailto--by passing to system API; handlers for Mac, Windows, and Unix

 if { [string equal [tk windowingsystem] aqua]} {
  if {[string match {http:*} $href]} {
	eval exec open $href &
	return {}
    }


    if {[string match {https:*} $href]} {
	eval exec open $href &
	return {}
    }



    if {[string match {ftp:*} $href]} {
	eval exec open $href &
	return {}
    }
    
    if {[string match {mailto:*} $href]} {
	eval exec open $href &
	return {}
    }
    
}

 if { [string equal [tk windowingsystem] x11]} {
    
     if {[string match {http:*} $href]} {
	   set launchfile [tk_getOpenFile -initialdir / -title "Please choose a program to launch $href:"]
	    eval exec $launchfile "$href"
    
	return {}
    }


    if {[string match {https:*} $href]} {
	  set launchfile [tk_getOpenFile -initialdir / -title "Please choose a program to launch $href:"]
	    eval exec $launchfile "$href"
	return {}
    }



    if {[string match {ftp:*} $href]} {
	  set launchfile [tk_getOpenFile -initialdir / -title "Please choose a program to launch $href:"]
	    eval exec $launchfile "$href"
	return {}
    }
    
    if {[string match {mailto:*} $href]} {
	  set launchfile [tk_getOpenFile -initialdir / -title "Please choose a program to launch $href:"]
	    eval exec $launchfile "$href"
	return {}
    }
 }
    
    if { [string equal [tk windowingsystem] win32]} {
	
	 if {[string match {http:*} $href]} {
	    eval exec [auto_execok start] \"\" $href &
    
	return {}
    }


    if {[string match {https:*} $href]} {
	eval exec [auto_execok start] \"\" $href &
	return {}
    }



    if {[string match {ftp:*} $href]} {
	eval exec [auto_execok start] \"\" $href &
	return {}
    }
    
    if {[string match {mailto:*} $href]} {
	eval exec [auto_execok start] \"\" $href &
	return {}
    }
    
}
	if {[string match #* $href]} {
		render $href
		return
	}
	if {[string match /* $href]} {
		set Url $href
	} else {
		set Url [file dirname $Url]/$href
	}
	update
	render $Url
}

# Supply an image callback function
# Read in an image if we don't already have one
# callback to library for display

proc HMset_image {win handle src} {
	global Url message
	if {[string match /* $src]} {
		set image $src
	} else {
		set image [file dirname $Url]/$src
	}
	set message "fetching image $image"
	update
	if {[string first " $image " " [image names] "] >= 0} {
		HMgot_image $handle $image
	} else {
		set type photo
		if {[file extension $image] == ".bmp"} {set type bitmap}
		catch {image create $type $image -file $image} image
		HMgot_image $handle $image
	}
}

# Handle base tags.  This breaks if more than 1 base tag is in the document

proc HMtag_base {win param text} {
	global Url
	upvar #0 HM$win var
	HMextract_param $param href Url
}

# downloading fonts can take a long time.  We'll override the default
# font-setting routine to permit better user feedback on fonts.  We'll
# keep our own list of installed fonts on the side, to guess when delays
# are likely

proc HMset_font {win tag font} {
	global message Fonts
	if {![info exists Fonts($font)]} {
		set Fonts($font) 1
		#.msg configure -fg blue
		set message "downloading font $font"
		update
	}
	#.msg configure -fg black
	set message ""
	catch {$win tag configure $tag -font $font} message
}

# Lets invent a new HTML tag, just for fun.
# Change the color of the text. Use html tags of the form:
# <color value=blue> ... </color>
# We can invent a new tag for the display stack.  If it starts with "T"
# it will automatically get mapped directly to a text widget tag.

proc HMtag_color {win param text} {
	upvar #0 HM$win var
	set value bad_color
	HMextract_param $param value
	$win tag configure $value -foreground $value
	HMstack $win "" "Tcolor $value"
}

proc HMtag_/color {win param text} {
	upvar #0 HM$win var
	HMstack $win / "Tcolor {}"
}

# Add a font size manipulation primitive, so we can use this sample program
# for on-line presentations.  sizes prefixed with + or - are relative.
#  <font size=[+-]3>  ..... </font>.  Note that this is not the same as
# Netscape's <font> tag.

proc HMtag_font {win param text} {
	upvar #0 HM$win var
	set size 0; set sign ""
	HMextract_param $param size
	regexp {([+-])? *([0-9]+)} $size dummy sign size
	if {$sign != ""} {
		set size [expr [lindex $var(size) end] $sign $size]
	}
	HMstack $win {} "size $size"
}

# This version is closer to what Netscape does

proc HMtag_font {win param text} {
	upvar #0 HM$win var
	set size 0; set sign ""
	HMextract_param $param size
	regexp {([+-])? *([0-9]+)} $size dummy sign size
	if {$sign != ""} {
		set size [expr [lindex $var(size) end] $sign  $size*2]
		HMstack $win {} "size $size"
	} else {
		HMstack $win {} "size [expr 10 + 2 * $size]"
	}
}

proc HMtag_/font {win param text} {
	upvar #0 HM$win var
	HMstack $win / "size {}"
}


proc makeImage {} {

    #KW: images taken from BlueSphere Icon Theme, http:://svgicons.sourceforge.net, released under BSD license

    image create photo back -data {

	R0lGODlhEAAQAOYAAAAAABBwiABogBiYqBCAoAh4mEDAyEhwiEiYqCiImPDw\
	    8DiIoLDI0FB4kGi4wCiYqDCouJjAyDiQoECIoJC4wHC4wECImECQoEiIoECY\
	    qFB4iFiAmCCYqGCgqFh4kKjI0BB4mAiAmDB4kFCAmAhwkKDAyBh4kCh4kABw\
	    kBiImDiAkEh4iCCQqDiImBCAmDCAmAB4kAh4kBB4kABwiABoiBCIoAiIoDCI\
	    oCB4mEjI2Pj4+Njg4CCYsGiwuGiosFDQ2BiIoDCQqCiouPD4+BiQqKjIyFjo\
	    6DC4yGDg6DiwwEjQ2MDQ2GCosCiAmCB4kEh4kEiAkAAAAAAAAAAAAAAAAAAA\
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAFEA\
	    LAAAAAAQABAAAAe1gFGCg4SFhg0YFxMWGoaDGQ88PBxAQBOOQUIGBkIcD0Qh\
	    TYUSQjlISj0+HCxEMDeDDSxJP0YVOhE2RDUEMQeCFgMQRw4KOkUSCAkuKCqC\
	    Cxw8TMU6OsUlKCQngjcDQCU6Q9TUDDMo2lEtNRwpDNQ7ER8dKDMigk8FlS7t\
	    FOXYNCuDbhRIUSPGkmsgYgjAQciDCRIuCsgwEYOEgACNCG1ogqLjDAECnHhw\
	    FGXECyc4REAhyZJQIAA7

    }


    image create photo depart -data {

	R0lGODlhEAAQANUAADgAAPgAAAAAAMAACPhYWMgICLAAAPiQkPgoKLhISLAw\
	    MPhAQMAAALAICPhQULgACLAACKgAAPiAgPi4uJAAAPB4ePiIiPioqPjAwPho\
	    aNBgYPigoPiwsEAAAPgYGPg4OPjY2PiYmPjg4NgYIPggIPgQEPgwMPhwcPjw\
	    8PgICPAAAPj4+Ph4eFAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAC4AIf/8SUNDUkdCRzEw\
	    MTIAAALMYXBwbAIAAABtbnRyUkdCIFhZWiAH1gABABQAAAAAAABhY3NwQVBQ\
	    TAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA9tYAAQAAAADTLWFwcGwAAAAA\
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAxy\
	    WFlaAAABFAAAABRnWFlaAAABKAAAABRiWFlaAAABPAAAABR3dHB0AAABUAAA\
	    ABRjaGFkAAABZAAAACxyVFJDAAABkAAAAA5nVFJDAAABoAAAAA5iVFJDAAAB\
	    sAAAAA52Y2d0AAABwAAAADBu/2RpbgAAAfAAAAA4ZGVzYwAAAigAAAB0Y3By\
	    dAAAApwAAAAtWFlaIAAAAAAAAHRLAAA+HQAAA8xYWVogAAAAAAAAWnMAAKym\
	    AAAXJlhZWiAAAAAAAAAoGAAAFVcAALgzWFlaIAAAAAAAAPNRAAEAAAABFsxz\
	    ZjMyAAAAAAABDEIAAAXe///zJgAAB5MAAP2Q///7ov///aMAAAPcAADAbmN1\
	    cnYAAAAAAAAAAQI5AABjdXJ2AAAAAAAAAAECOQAAY3VydgAAAAAAAAABAjkA\
	    AHZjZ3QAAAAAAAAAAQABAAAAAAAAAAEAAAABAAAAAAAAAAEAAAABAAAAAAAA\
	    AAEAANxuZGluAAAAAAAAADAAAKFIAABXCgAAS4UAAJrhAAAnrgAAE7YAAFAN\
	    AABUOQACOOQAAjjkAAI45GRlc2MAAAAAAAAAGkNhbGlicmF0ZWQgUkdCIENv\
	    bG9yc3BhY2UAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAdGV4\
	    dAAAAABDb3B5cmlnaHQgQXBwbGUgQ29tcHV0ZXIsIEluYy4sIDIwMDUAAAAA\
	    ACwAAAAAEAAQAAAGkkCXcEgsFlkSi9JSMQotnAkGM+FsEkbOJbS0sE4nDdET\
	    kpw2LFbGSyAohiaL47DCOE4okGMx6ghJew4TKxciKycfCAV+Lh4kKQgkGyso\
	    Dh4eAQyMJSUBAR4gKCsSngEPjAEpASYiKCYSKyGeDYwqniQYC55cswBDEaXB\
	    ngMtRQbCw8VGFA3NzRDKTh3T075O10VBADs=



    }

    image create photo home -data {

	R0lGODlhEAAQAOYAAAAAAFAAEBAQGJiQkIAoMDBIUKioqDhISEhYWIAgMOCg\
	    OJAoONDI0LgwWNCIIEhgYMhAYJhwaLCIeMCgmMCoqOCwcMiQQJCIiKCYkGgI\
	    EIgoMMCooIgwKJBQWJhgYLi4sGAQGKgIMLBgALCwsHAQIGgYIPjIUNjY2NCQ\
	    KNDAwPjIeOCoUHAYIFAgGLAgQLgwUMiQMPC4SPjAYMA4WLCAYDhIUPDAYNDQ\
	    2Mi4uNDY2MjIyNigSIhQGNCIGMiIKNioYKhoOLAQMMBAWEBYWNiIEDBAQCg4\
	    QPjAQPjQaMjI0GAACBAYIOC4gPjAWLBoIPjYmNDQ0KAQMKgwQPC4cJhAINjg\
	    4PjQgOjAgODg4ODo6Ojw8PDw8Pj4+AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAF0A\
	    LAAAAAAQABAAAAeegF2Cg4SFhS2CICSGhVM8AUIvGYyDFlQNLlFBk5QcLwsd\
	    HgkhnIUgMxoYWlkfo6WCJRAaA1lcW1gjBKSDLLIXOSkSEQw5BgSbglIkB0k4\
	    NFcrQDpQCEqvBVATTE9IKBQnQ4YHORMVVjIoGycIhgU3Ez8qTQ7q7IU15Ds2\
	    MT314uQKTBwhou6BIXwTnMDwIYJCFYPtjAhYskRAkQKEAgEAOw==

    }
}

proc showDoc {file} {

    #KW: finally, let's render the page
    global Size Indent Home Url Running message HMlast datadir

    set HMlast ""

    makeImage
    setupBrowser		;# the catch lets us re-source this file
    #set Size 0					;# font size adjustment
    set Size -1
    #set Indent 1.2				;# tab spacing (cm)
    set Indent 1

    set Home [file join [file dirname [info script]] html index.html];# home document
    #set Home $file
    #set Url $Home				;# current file
   # set Running busy			;# page status
   # set message ""				;# message line

    #puts "File name is $Home"

    # make the interface and render the home page
    
    HMinit_win .help.text
    HMset_state .help.text -size $Size
    HMset_indent .help.text $Indent
    #wm withdraw .help
    #render $Home
    #set Home $file
    render $file


}
